﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Runtime.InteropServices;
using System.Security.Cryptography.X509Certificates;
using System.Windows.Media;
using Ascon.Pilot.SDK.XpsDigitalSigner.Properties;
using Ascon.Pilot.Theme.ColorScheme;

namespace Ascon.Pilot.SDK.XpsDigitalSigner
{
    [Export(typeof(IObjectContextMenu))]
    public class XpsDigitalSignerPluginSample : IObjectContextMenu
    {
        private const string SignDocumentFromPlugin = "SignDocumentFromPlugin";
        private readonly IObjectsRepository _repository;
        private readonly IXpsDigitalSigner _digitalSigner;
        private IDataObject _xpsDoc;
        private List<IDataObject> _selectedObjects;
        private readonly IObservable<INotification> _signatureNotifier;
        private readonly IObservable<INotification> _fileNotifier;

        [DllImport("user32.dll")]
        public static extern IntPtr GetForegroundWindow();

        [ImportingConstructor]
        public XpsDigitalSignerPluginSample(IObjectsRepository repository, IXpsDigitalSigner digitalSigner)
        {
            ColorScheme.Initialize(Color.FromArgb(255, 78, 156, 201));
            _digitalSigner = digitalSigner;
            _repository = repository;
            _signatureNotifier = _repository.SubscribeNotification(NotificationKind.ObjectSignatureChanged);
            _fileNotifier = _repository.SubscribeNotification(NotificationKind.ObjectFileChanged);
        }

        public void OnMenuItemClick(string itemName)
        {
            SignDocument();
        }

        private void SignDocument()
        {
            var certificate = SelectCertificate();
            var batchSigner = new BatchSigner(_selectedObjects, _repository.GetCurrentPerson().Positions.Select(p => p.Position).ToList(), certificate, _digitalSigner);
            var model = new ProgressDialogModel(batchSigner, _signatureNotifier, _fileNotifier, _repository);
            model.Run();
        }
        
        private static X509Certificate2Collection GetCertificatesCollection()
        {
            var store = new X509Store(StoreLocation.CurrentUser);
            store.Open(OpenFlags.OpenExistingOnly | OpenFlags.ReadOnly);
            var certificateCollection = store.Certificates;
            store.Close();

            certificateCollection = certificateCollection.Find(X509FindType.FindByTimeValid, DateTime.Now, true);
            certificateCollection = certificateCollection.Find(X509FindType.FindByKeyUsage, X509KeyUsageFlags.DigitalSignature, false);
            var withPrivate = new X509Certificate2Collection();
            foreach (X509Certificate2 cer in certificateCollection.Cast<X509Certificate2>().Where(cer => cer.HasPrivateKey))
                withPrivate.Add(cer);
            certificateCollection = withPrivate;

            return certificateCollection;
        }

        private X509Certificate2 SelectCertificate()
        {
            var certificateCollection = GetCertificatesCollection();

            if (certificateCollection.Count == 0)
                throw new Exception(Resources.NoSigningCertificateAvaliable);

            if (certificateCollection.Count == 1)
                return certificateCollection[0];

            certificateCollection = X509Certificate2UI.SelectFromCollection(
                certificateCollection,
                Resources.ChoosingCertificate,
                Resources.NeedChooseCertificate,
                X509SelectionFlag.SingleSelection);

            if (certificateCollection.Count == 0)
            {
                throw new Exception(Resources.NeedChooseCertificate);
            }
            return certificateCollection[0];
        }

        public void BuildContextMenu(IMenuHost menuHost, IEnumerable<IDataObject> selection, bool isContext)
        {
            if (isContext || selection == null)
                return;

            _selectedObjects = selection.ToList();
            _xpsDoc = selection.FirstOrDefault();
            if (_xpsDoc != null)
                menuHost.AddItem(SignDocumentFromPlugin, Resources.PluginName, null, 0);
        }
    }
}
