﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Threading.Tasks;
using Ascon.Pilot.Common;
using Ascon.Pilot.DataClasses;
using Ascon.Pilot.DataModifier;
using Ascon.Pilot.ServerExtensions.SDK;

namespace ServerActivitySample
{
    [Export(typeof(IServerActivity))]
    public class UpdateDocumentStateOnSigning : IServerActivity
    {
        public string Name => "UpdateDocumentStateOnSigning";
        public const string StateAttributeName = "stateAttributeName";
        public const string SignedStateName = "signedStateName";
        public const string NotSignedStateName = "notSignedStateName";
        
        public Task RunAsync(IModifierBase modifier, IModifierBackend backend, IServerActivityContext serverActivityContext, IAutomationEventContext automationEventContext)
        {
            var source = automationEventContext.Source;
            var isFullySigned = IsFullySigned(source);
            if(!serverActivityContext.Params.TryGetValue(StateAttributeName, out var stateAttributeName))
                throw new InvalidOperationException($"{StateAttributeName} parameter is not defined in UpdateDocumentStateOnSigning activity");
            if(!serverActivityContext.Params.TryGetValue(SignedStateName, out var signedStateName))
                throw new InvalidOperationException($"{SignedStateName} parameter is not defined in UpdateDocumentStateOnSigning activity");
            if(!serverActivityContext.Params.TryGetValue(NotSignedStateName, out var notSignedStateName))
                throw new InvalidOperationException($"{NotSignedStateName} parameter is not defined in UpdateDocumentStateOnSigning activity");
            
            var signedState = backend.GetMetadata().UserStates.FirstOrDefault(x => x.Name == (string)signedStateName);
            if(signedState == null)
                throw new InvalidOperationException($"State {signedStateName} not found");

            var notSignedState = backend.GetMetadata().UserStates.FirstOrDefault(x => x.Name == (string)notSignedStateName);
            if(notSignedState == null)
                throw new InvalidOperationException($"State {NotSignedStateName} not found");

            var newStateId = isFullySigned ? signedState.Id : notSignedState.Id;
            if (!source.Attributes.TryGetValue((string)stateAttributeName, out var stateAttributeValue) || stateAttributeValue.GuidValue != newStateId)
            {
                modifier.EditObject(source).SetAttribute((string)stateAttributeName, newStateId);
            }

            return Task.CompletedTask;
        }

        public static bool IsFullySigned(INObject @object)
        {
            var xpsFile = @object.ActualFileSnapshot.Files.FirstOrDefault(f => FileExtensionHelper.IsXpsAlike(f.Name));
            if (xpsFile == null || xpsFile.Signatures.Count == 0)
                return false;
            var requestCount = xpsFile.Signatures.Count;
            var signCount = xpsFile.Signatures.Count(signature => !string.IsNullOrEmpty(signature.Sign));
            return signCount == requestCount;
        }
    }
}
